;;
;;  To be run as follows, from the scheme root
;;
;;  scheme -no-init-file -band runtime.com -heap 3000 -load build.scm -eval XXXX
;;
;;  where XXXX is (edwin.com)     to build edwin
;;                (compiler.com)  to build compiler
;;                (all.com)       to build edwin+compiler
;;

(define edwin.com)
(define compiler.com)
(define all.com)
(define 6001.com)

;; Reduce the effects of paging

((make-primitive-procedure 'WIN32-FLUSH-OLD-HALFSPACE-AFTER-GC?!) #T)

(let ()

  (define (have symbol)
    (not (lexical-unreferenceable? (the-environment) symbol)))

  (define (do-on-runtime loader band-name)
    (if (or (have 'edwin) (have 'sf))
	(error (string-append "Started with wrong band: can only build "
			      band-name  " on runtime.com"))
	(begin (loader)
	       (result-band band-name))))

  (define (do-edwin.com)    (do-on-runtime load-edwin    "edwin.com"))
  (define (do-compiler.com) (do-on-runtime load-compiler "compiler.com"))
  (define (do-6001.com)     (do-on-runtime load-6001     "6001.com"))

  (define (do-all.com)
    (if (not (have 'edwin))  (load-edwin))
    (if (not (have 'cf))     (load-compiler))
    (result-band "all.com"))

  (define (result-band name)
    ;; unsetting the variables makes sure that this build garbage will not
    ;; survive long after the band is reloaded
    (set! edwin.com)
    (set! all.com)
    (set! compiler.com)
    (set! 6001.com)
    (display "\n\nDumping band ") (display (in-library name)) (display "...\n")
    (disk-save (in-library name))
    (display "\n\nDone.\n\n")
    (%exit))

  (define (in-library filename)
    (merge-pathnames filename (system-library-pathname "")))

  (define (load-edwin)
    (fluid-let ((flush-purification-queue! (lambda () 42)))
      (load (in-library "eddel.com"))
      (display "\n\nDumping doc strings...\n\n")
      ((access dump-doc-strings (->environment '(edwin)))
       (in-library "edwin/etc/eddocstr.dat")
       "eddocstr.dat"))
    (flush-purification-queue!))

  (define (load-compiler)
    (load (in-library "compdel.com")))

  (define (load-6001)
    (load-edwin)
    (load (in-library "6001del.com")))

  (define (with-error-report thunk)
    (lambda ()
      (bind-condition-handler (list condition-type:error)
	  (lambda (condition)
	    (display "\n\n\n\nBuild failed:  ") (display condition)
	    (display "\n\n")
	    (write-condition-report condition (current-output-port))
	    (display "\n\nType (%exit 1) to quit Scheme.\n"))
	thunk)))

  (set! edwin.com    (with-error-report do-edwin.com))
  (set! all.com      (with-error-report do-all.com))
  (set! compiler.com (with-error-report do-compiler.com))
  (set! 6001.com     (with-error-report do-6001.com)))
