;;
;;  To be run as follows:
;;
;;  scheme -no-init-file -compiler -heap 3000 -load rebuild.scm -eval XXXX
;;
;;  where XXXX is (do.eddel)     to build eddel.com
;;                (do.compiler)  to build compiler
;;                (do.all)       to build edwin+compiler
;;

(define edwin.com)
(define compiler.com)
(define all.com)

(let ()

  (define (have symbol)
    (not (lexical-unreferenceable? (the-environment) symbol)))

  (define (do-on-runtime loader band-name)
    (if (or (have 'edwin) (have 'sf))
	(error (string-append "Started with wrong band: can only build "
			      band-name  " on runtime.com"))
	(begin (loader)
	       (result-band band-name))))

  (define (do-edwin.com)    (do-on-runtime load-edwin    "edwin.com"))
  (define (do-compiler.com) (do-on-runtime load-compiler "compiler.com"))

  (define (do-all.com)
    (if (not (have 'edwin))  (load-edwin))
    (if (not (have 'cf))     (load-compiler))
    (result-band "all.com"))

  (define (result-band name)
    ;; unsetting the variables makes sure that this build garbage will not
    ;; survive long after the band is reloaded
    (set! edwin.com)
    (set! all.com)
    (set! compiler.com)
    (display "\n\nDumping band ") (display (in-library name)) (display "...\n")
    (disk-save (in-library name))
    (display "\n\nDone.\n\n")
    (%exit))

  (define (in-library filename)
    (merge-pathnames filename (system-library-directory-pathname "")))

  (define (load-edwin)
    (fluid-let ((flush-purification-queue! (lambda () 42)))
      (load (in-library "eddel.com")))
    (display "\n\nDumping doc strings...\n\n")
    ((access dump-doc-strings (->environment '(edwin)))
     (in-library "edwin/etc/eddocstr.dat")
     "eddocstr.dat")
    (flush-purification-queue!))

  (define (load-compiler)
    (load (in-library "compdel.com")))

  (define (with-error-report thunk)
    (lambda ()
      (bind-condition-handler (list condition-type:error)
	  (lambda (condition)
	    (display "\n\n\n\nBuild failed:\n\n")
	    (write-condition-report condition (current-output-port))
	    (display "\n\nType (exit) to quit Scheme.\n"))
	thunk)))

  (set! edwin.com    (with-error-report do-edwin.com))
  (set! all.com      (with-error-report do-all.com))
  (set! compiler.com (with-error-report do-compiler.com)))
